use reqwest::blocking as reqwest;
use crate::errors::Error;
use std::collections::HashMap;
use std::convert::TryFrom;

mod structs;
use structs::{Street, Date, PickupTime};


/// The base URL where their API starts
const BASE_URL: &str = "https://www.ivago.be/nl/particulier/afval/ophaling";
const SEARCH_URL: &str ="https://www.ivago.be/nl/particulier/autocomplete/garbage/streets";


/// Searches the Ivago API for streets in the given city
///
/// # Arguments
///
/// * `street` - name of the street
/// * `city` - city the street is in
// TODO find out how to do this async
pub fn search_streets(street_name: String) -> Result<Vec<Street>, Error> {
    let client = reqwest::Client::new();
    let response = client.get(SEARCH_URL).query(
        &[("street", &street_name)]).send();

    // Get the request, and handle failed requests
    let response = match response {
        Ok(res) => res,
        Err(error) => return Err(Error::BackendError(
            String::from("Ivago API failed to respond."))),
    };

    // Get the JSON
    let data: Vec<HashMap<String, String>> = match response.json() {
        Ok(res) => res,
        Err(error) => return Err(Error::BadResponseError(
                String::from("Ivago API responsed with bad JSON.")
                )),
    };

    let mut output: Vec<Street> = Vec::new();

    // We iterate over every item and extract the needed data
    for map in data.iter() {
        if let Some(value) = map.get("value") {
            match Street::try_from(value) {
                Ok(street) => output.push(street),
                Err(_) => continue,
            }
        }
    }

    Ok(output)
}


/// Return the known pickup times for the given street and/or city
///
/// # Arguments
///
/// * `street` - name of the street
/// * `city` - city the street is in
pub fn get_pickup_times(street: Street) -> Vec<PickupTime> {
    Vec::new()
}
